#include <sys/heap.h>
#include <bt/bt_defs.h>
#include <bt/bt_hci_defs.h>
#include <bt/bt_msg_defs.h>

// creates a que of acl pkt buffers
bt_msg_fifo* bt_msg_fifo_create(void)
{
	bt_msg_fifo* fifo;
	
	// dyn. allocate msg buffer
	fifo = NutHeapAllocClear(sizeof(bt_msg_fifo));
	fifo->size = 0;
	fifo->first = NULL;
	fifo->last = NULL;
	return fifo;
}


void bt_msg_fifo_destroy(bt_msg_fifo* fifo)
{
	bt_msg* msg;
	
	while ((msg = fifo->first)) {
		// remove msg from queue
		fifo->first = msg->next;
		// destroy the msg
		bt_acl_pkt_buf_destroy(msg);
	}
	
	// free memory
	NutHeapFree(fifo);
}

bt_msg* bt_msg_fifo_get_first(bt_msg_fifo* fifo)
{
	bt_msg* msg;
	
	// remove first free buffer from the queue of free bufs
	msg = fifo->first;
	if (msg) {
		fifo->first = msg->next;
		fifo->size--;
		msg->next = NULL;
	}
	return msg;
}

void bt_msg_fifo_add_first(bt_msg_fifo* fifo, bt_msg* msg)
{
	// add fragment passed to the top of the buffer
	msg->next = fifo->first;
	fifo->first = msg;
	// if the buffer was empty, we have to set the pointer to the last buffered
	// fragment
	if (!fifo->size) fifo->last = msg;
	// increment size of the buffer
	fifo->size++;
}

void bt_msg_fifo_add_last(bt_msg_fifo* fifo, bt_msg* msg)
{
	// if the queue is empty we add the buffer to the head of the queue
	if (!fifo->size)
		return bt_msg_fifo_add_first(fifo, msg);
	// else, we add the buffer to the end of the queue
	fifo->last->next = msg;
	fifo->last = msg;
	msg->next = NULL;
	// increment size of the queue
	fifo->size++;
}
